/*****************************************************************************
 *          Copyright: (C) Copyright David Goodwin, 2014, 2015, 2016
 *            License: GNU General Public License
 *****************************************************************************
 *
 *   This is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ****************************************************************************/

/*
 * Data Source for the PC Sensor TEMPerHUM 1F_H1 Temperature+Humidity Sensor
 *   This is *NOT* compatible with all TEMPerHUM devices. Just the 1F_H1 and
 *   anything else sufficiently identical to it.
 */

#include "temperhumdatasource.h"

#include <QtDebug>
#include <QThread>

unsigned const char cmd_temp[9] = { 0x00, 0x01, 0x80, 0x33, 0x01, 0x00, 0x00, 0x00, 0x00 };
unsigned const char cmd_vers[9] = { 0x00, 0x01, 0x86, 0xFF, 0x01, 0x00, 0x00, 0x00, 0x00 };
unsigned const char cmd_calb[9] = { 0x00, 0x01, 0x82, 0x77, 0x01, 0x00, 0x00, 0x00, 0x00 };

const int VENDOR_ID  = 0x0C45;
const int PRODUCT_ID = 0x7402;

#define READ_BUFFER_SIZE 9

TemperHumDataSource::TemperHumDataSource(float temperatureCalibrationOffset,
                                         float humidityCalibrationOffset,
                                         QObject *parent) :
    AbstractDataSource(parent)
{
    handle = NULL;
    collecting = false;

    this->temperatureCalibrationOffset = temperatureCalibrationOffset;
    this->humidityCalibrationOffset = humidityCalibrationOffset;

    timer.reset(new QTimer(this));
    connect(timer.data(), SIGNAL(timeout()), this, SLOT(collectData()));
}

TemperHumDataSource::~TemperHumDataSource() {
    stopCollecting();
    closeDevice();
}

bool TemperHumDataSource::openDevice() {
    //int res = hid_init();
    //qDebug() << "INIT:" << res;

    handle = hid_open(VENDOR_ID, PRODUCT_ID, NULL);

    if (handle == NULL) {
        qDebug() << "Device open failed.";
        emit error(tr("Error"),
                   tr("Failed to open USB HID sensor device. Check device is "
                   "connected\nand restart application."));
        return false;
    }

    temp_hum_reading_t calibration = getCalibrationValue();
    qDebug() << calibration.temperature << calibration.humidity;

    QString version = getVersion();
    qDebug() << version;

    AbstractDataSource::openDevice();
    return true; // Ready for work.
}

bool TemperHumDataSource::closeDevice() {
    if (handle != NULL)
        hid_close(handle);
    collecting = false;
    return true;
}

void TemperHumDataSource::collectData() {

    temp_hum_reading_t result = getReading();

    emit newTempHumData(result.temperature, result.humidity);
}

// TODO: Is this really useful? There seems to be no way of setting
// the value stored on the device and by default none of the three
// sensors I have actually have a pre-set value.
temp_hum_reading_t TemperHumDataSource::getCalibrationValue() const {
    unsigned char read_buffer[9];
    double calibration_value = 0, humidity_calibration_value = 0;

    hid_write(handle, cmd_calb, sizeof(cmd_calb));
    QThread::usleep(100);
    hid_read(handle, read_buffer, 9);

    if (read_buffer[0] == 130) {
        calibration_value = read_buffer[2];
        calibration_value *= (1.0/16.0);
        qDebug() << "Calibration: " << calibration_value;
    }
    QThread::usleep(100);

    if (read_buffer[0] == 130) {
        double calibration_value;
        double humidity_calibration_value;
        if ((int)read_buffer[2] < 128) {
            double num2 = (double)((int) read_buffer[2] * 256 + (int)read_buffer[3]);
            calibration_value = num2 / 100.0;
        } else {
            double num2 = -(double)(65536 - (int) read_buffer[2] * 256 - (int) read_buffer[3] / 100.0);
            calibration_value = num2;
        }

        if ((int)read_buffer[4] < 128) {
            humidity_calibration_value = (double)((int)read_buffer[4] * 256 + (int)read_buffer[5] / 100.0);
        } else {
            humidity_calibration_value = -(double)(65536 - (int)read_buffer[4] * 256 - (int)read_buffer[5]) / 100.0;
        }
        printf("Calibration: %f %f\n", calibration_value, humidity_calibration_value);
    }

    temp_hum_reading_t result;
    result.temperature = calibration_value;
    result.humidity = humidity_calibration_value;

    return result;
}

QString TemperHumDataSource::getVersion() const {
    unsigned char read_buffer[READ_BUFFER_SIZE];

    memset(read_buffer, 0, sizeof(char)*READ_BUFFER_SIZE);

    QString result;

    hid_write(handle, cmd_vers, sizeof(cmd_vers));
    QThread::usleep(100);
    hid_read(handle, read_buffer, 9);
    qDebug() << "Version read 1:" << read_buffer;
    result = QString::fromLocal8Bit((char*)read_buffer, 9);
    QThread::usleep(100);

    hid_read(handle, read_buffer, 9);
    qDebug() << "Version read 2:" << read_buffer;
    QThread::usleep(100);


    return result;
}



temp_hum_reading_t TemperHumDataSource::getReading() const {
    temp_hum_reading_t reading;
    float temperature, humidity, working_1, working_2;
    unsigned char read_buffer[READ_BUFFER_SIZE];

    memset(read_buffer, 0, sizeof(char)*READ_BUFFER_SIZE);

    hid_write(handle, cmd_temp, sizeof(cmd_temp));
    QThread::usleep(50);
    hid_read(handle, read_buffer, 9);

    temperature = (double) ((int) read_buffer[2] * 256 + (int) read_buffer[3]) * 0.01 - 40.0;

    // Humidity sensor readout
    working_1 = (float) ((int)(read_buffer[4]) * 256 + (int) read_buffer[5]);

    // Calculations below come from the Sensirion SHT1x datasheet

    // RH Linear
    working_2 = -2.0468 + 0.0367 * working_1 + -1.5955e-6 * working_1 * working_1;

    // Temperature compensation for humidity for 12bit readings
    humidity = (temperature - 25.0) * (0.01 + 0.00008 * working_1) + working_2;
    if (humidity > 99.0)
      humidity = 100.0;
    if (humidity < 0.1)
      humidity = 0.1;

    reading.temperature = temperature;
    reading.humidity = humidity;

    qDebug() << "Temp:" << reading.temperature << "\tHum:" << reading.humidity;

    reading.temperature += temperatureCalibrationOffset;
    reading.humidity += humidityCalibrationOffset;

    return reading;
}

void TemperHumDataSource::setInterval(int interval) {
    timer->setInterval(interval);
}

int TemperHumDataSource::getInterval() const {
    return timer->interval();
}

void TemperHumDataSource::startCollecting() {
    if (deviceOpen) {
        timer->start();
        collecting = true;
    }
}

void TemperHumDataSource::stopCollecting() {
    timer->stop();
    collecting = false;
}

bool TemperHumDataSource::isCollecting() {
    return collecting;
}
